// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011, 2013
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
def createServer(server, owner = false) {
	server.jmxport = '2809';
    System.err.println("createServer: " + [server, owner].toString());
    
    if (server.alias == '') {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.name.required', null);
    }
    if (server.alias == "\\") {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.singleslash', null);
    }
    
    if (server.name == '') {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.hostname.required', null);
    }
    if (server.name == "\\") {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.singleslash', null);
    }
    
    def filters = [alias:server.alias, exactMatch:true];
    def servers = getServers(filters);
    if (servers.size() > 0) {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.alreadyexists', null, [server.alias]);
    }

    def resource = createResource('servers', server, owner);
    if (resource.errorMessage != null && resource.errorMessage != '') {
    	if (resource.errorMessage =~ /length/) {
    		def lengthValue = resource.errorMessage.split("length ");
    		return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.property.length', null, [lengthValue[1]]);
    	}
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.create', null, [resource.errorMessage]);
    } else {
        return resource;
    }
}

def updateServer(server) {
    server.jmxport = '2809';
    if (server.alias != null) {
        // If server alias is empty throw error because it is required.
        if (server.alias == '') {
            return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.name.required', null);
        }
        if (server.alias == "\\") {
            return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.singleslash', null);
        }
        

        // Check if server alias is already used by another server.
        def filters = [alias:server.alias, exactMatch:true];
        def filteredServers = getServers(filters);
        
        // Check each server that matched the filter search and if the match
        // is this current server, skip it because we don't care about duplicating
        // ourself.
        filteredServers.each { filteredServer ->
            if (filteredServer.id.toString() != zget("/request/params/serversId").toString()) {
                return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.alreadyexists', null, [server.alias]);
            }
        }
    }

    // If server name is empty throw error because it is required.
    if (server.name != null) {
       if (server.name == '') {
           return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.hostname.required', null);
       }
       if (server.name == "\\") {
           return com.ibm.websphere.rest.RenderUtils.renderError(400, 'error.catalogserver.singleslash', null);
       }
    }
    

    def resource = updateResource('servers', server,  ["id", "alias", "name", "listenerport", "jmxport"]);
    return resource;
}

def deleteServer(id) {
    def rv   = deleteResource('servers', id)
    def refs = getResources('domainsservers', [serverid: id], false)

    refs.each { 
        deleteResource('domainsservers', it.id)
    }
    
    return rv
}

def getServer(id) {
    def rv = getResource('servers', [id: id], ["id", "alias", "name", "listenerport", "jmxport"])
    System.err.println("getServer: " + rv.toString() )
    return rv
}

def getServers(filters, owner = false) {
    def _filters = filters.clone()
    def rv = getResources( 'servers', _filters, false,  ["id", "alias", "name", "listenerport", "jmxport"])
    rv.each { server ->
    	server.overrideListViewLabel = "alias";
    }
    System.err.println("getServers: " + [filters, rv].toString())

    return rv
}