/* TODO: This should move to a xs.app.setup, so it is easier for us to not ship this. */
import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import groovy.rainmaker.constants.*;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.io.IOException;



def taskLogger;
def lang;

//
// This method will update the classpath that each subsequent zero process will run with. It does so
// by updating the resolved.properties file.
//
def setupClasspath() {
    def updateResolvedProperties = false;
    def p = new Properties()
    def f = new File(config.dependencies.'ibm:xs.app'[] + "/.zero/private/resolved.properties")
    def fs = new FileInputStream(f.absolutePath)
    p.load(fs)
    
    def lib = new File(config.dependencies.'ibm:xs.app'[] + "/../lib/ogclient.jar")
    
    if ( lib.exists() ) { //NON-WAS installation
		if (!p["classpath.cli"].contains(lib.absolutePath)) {
			p["classpath.cli"] += System.getProperties().getProperty("path.separator") + lib.absolutePath
			updateResolvedProperties = true;
		}
    } else {
		//We need to load the runtime jar because it has a class for FFDCSelfIntrospectable that is needed by console start.  That jar 
		//is not in the wsogclient.jar, which the above if check adds the ogclient.ar to handle this case for standalone
		def lib_was = new File(config.dependencies.'ibm:xs.app'[] + "/../../../plugins/com.ibm.ws.runtime.jar")
		if (!p["classpath.cli"].contains(lib_was.absolutePath)) {
			p["classpath.cli"] += System.getProperties().getProperty("path.separator") + lib_was.absolutePath
			updateResolvedProperties = true;
		}
	}
    
    // On Windows shorten paths to be relative so we have a better chance
    // at staying below 8096 characters.  This is a limit on some versions of
    // windows.
    def classPath = p["classpath.cli"];
    if( System.getProperty("os.name").indexOf("Windows") >= 0 ) {
        classPath = shortenWindowsPath(classPath);
        p["classpath.cli"] = classPath;
        updateResolvedProperties = true;
    }
    
    if (updateResolvedProperties) {
        def os = new FileOutputStream(f.absolutePath)
        p.store(os, "")
    }
}   

def shortenWindowsPath(classPath) {
    return classPath.replace(config.dependencies.'ibm:xs.app'[], "..\\console");
}

def broadcastInfo(message) {
    Output.info(message);
    log(message);
}

def broadcastError(message) {
    Output.error(message);
    log(message);
}

def log(message) {
    taskLogger.append(getNow()+" :: "+message+"\n");
}

def getNow() {
    SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
    return formatter.format(Calendar.getInstance().getTime());
}


def onCliTask() {


    taskLoggerPath = config.dependencies.'ibm:xs.app'[] + "/logs/startConsoleServer.log"
    lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
    taskLogger = new File(taskLoggerPath);

    if (taskLoggerPath.indexOf("\\") != -1) {
        taskLoggerPath = taskLoggerPath.replace("\\", "\\\\").replace("/", "\\\\");
    }

    try {
        socket = new Socket();
        SocketAddress address = new InetSocketAddress('localhost', config.https.port[].toInteger());
        socket.connect(address, 1000);
        broadcastInfo(MessageUtils.message(lang, 'XSW0166'));
        
        //
        // The HTTP start file is from this process (it is still running). We need
        // to rename it back.
        //
        httpsfile(true);
        
        config.exitCode = 1
        return
    } catch (IOException e) {
		//do nothing, this means that the console is not running and we want to proceed with the console startup
    } finally {
        if (socket != null) {
            try {
                socket.close();
            } catch (Exception e){}
        }
    }
    
    //
    // Update the zero classpath.
    //
    setupClasspath();
    
    //
    // Create XSCON's version of the java.endorsed.dirs. We will replace XC10's version below.
    //
    String javahome = new File(System.getProperty("java.home")).getAbsolutePath();
    if (!javahome.endsWith("jre")) javahome += File.separator + "jre";
    String endorsedDirs = config.dependencies.'ibm:xs.app'[] + "/../lib/endorsed"; 
    endorsedDirs += File.pathSeparator + javahome + "/lib/endorsed";
    

    //Write out the jvmargs to a file so that they can be appended to the ZERO_OPTS before starting smash
    vars = zget("/config/zso/jvmargs");
    new File("ZERO_OPTS.config").withWriter { out ->
        vars.each() { var ->
            if (var != "-Djava.util.logging.manager=com.ibm.ws.bootstrap.WsLogManager" && 
                var != "-Djava.util.logging.configureByServer=true" &&
                !var.startsWith("-Xdump") &&
                !var.startsWith("-Djava.endorsed.dirs")) {
                out.write(var+" ")
            } else if (var.startsWith("-Djava.endorsed.dirs")) {
                // Use our version of java.endorsed.dirs. The endorsed.dirs value may contain spaces
                // so surround with quotes.
                endorsedDirs = "\"" + endorsedDirs + "\"";
                out.write("-Djava.endorsed.dirs=" + endorsedDirs + " ");
            }
        }
    }
    
    //
    // The HTTPS start file was from a previous start since there was no console currently running.
    // Delete it.
    //
    httpsfile(false);
}

def httpsfile(keep) {
    def startFile      = new File((String) config.xsa.httpsStartFile[]);
    def startFileSaved = new File((String) config.xsa.httpsStartFile[] + ".save");

    if (startFileSaved.exists()) {
        if (keep) {
            startFile << startFileSaved.text;
        } else {
            startFileSaved.delete();
        }
    }
}

