/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;

import com.ibm.websphere.rainmaker.appliance.EthernetInterfaces;
import com.ibm.websphere.rest.RESTUtils;

def onList()
{ 
	if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    try
    {
        def interfaceNames = EthernetInterfaces.getEthernetInterfaces();
    
        def results = [];
        
        interfaceNames.each() { current ->
        		results.add(EthernetInterfaces.getEthernetInterface(current));
            }
        
        request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
        request.json.output = results;
        render();
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08000', t);
    }
}

def onRetrieve()
{
    def results = null;
    
    try
    {
        String id = request.params.ethernetInterfacesId[];
		def ethernetInterface = EthernetInterfaces.getEthernetInterface(id); 
		if(ethernetInterface!= null) { // Use ViewEngine JSON rendering 
			request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
			request.json.output = ethernetInterface;
			render();
		} else { // Error handling 
			request.status = HttpURLConnection.HTTP_NOT_FOUND; 
			request.error.message = "ethernetInterface $id not found."; 
			request.view = 'error'; 
			render(); 
		} 
        
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08049', t);
    }
}

def onUpdate()
{    
	if (!RESTUtils.authorized())
		 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');   
    def type = null;
    def json = null;
    
    try
    {
        json = Json.decode(request.input[]);
        if (!com.ibm.websphere.rest.RESTUtils.areValidParameters(json)){
              return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
		}
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    try
    {
        logger.FINEST { "Executing method update." };
         EthernetInterfaces.updateEthernetInterface(json);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08004', t);
    }
}
