/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;
import com.ibm.websphere.rainmaker.auditing.AuditDataHelper;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rest.RESTUtils;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.users.Users;
import groovy.rainmaker.constants.Action;

def onPutCollection()
{
    def session = user.session[];
    def userID = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
    
    //if (userID != session.id && !session.permissions.contains(Permissions.APPLIANCE_ADMIN))
    if ((userID != session.id) && !RESTUtils.authorized())
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    def change = null;
    def targetUser = null;
    
    try
    {
        change = Json.decode(request.input[]);
        if (!com.ibm.websphere.rest.RESTUtils.areValidParameters(change)){
              return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002');
		}
        targetUser = Users.getUser(userID);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    change.value = change.value.trim();
    change.verification = change.verification.trim();
    
    if (!change.value.equals(change.verification))
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08006');
    
    if (change.value.length() == 0)
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08017');
    
	logger.INFO { MessageUtils.message('RM08070', [session.username, targetUser.username]) };
    
    def throwable;
    def result;
    try
    {
        result = changePassword(userID, change.value);
    }
    catch (Throwable t)
    {
        throwable = t;
    }

    if(throwable || result < 0){
        if (result == -2) {
        	// Audit that the password failed to changed
        	AuditDataHelper.audit(Action.UPDATE, "password",
		        MessageUtils.message('RM08100', [targetUser.username]));
    
        	logger.SEVERE { MessageUtils.message('RM08100', [targetUser.username]) };
	        if(throwable){
                return com.ibm.websphere.rest.RenderUtils.renderError(500, MessageUtils.message('RM08100', [targetUser.username]), throwable);
    	    }else{
                return com.ibm.websphere.rest.RenderUtils.renderError(500, MessageUtils.message('RM08100', [targetUser.username]));
        	}
        } else if (result == -1) {
        	// Audit that the password changed, but the email did not get sent
        	AuditDataHelper.audit(Action.UPDATE, "password",
		        MessageUtils.message('RM08101', [targetUser.username]));
    
        	logger.SEVERE { MessageUtils.message('RM08101', [targetUser.username]) };
        	if(throwable){
                return com.ibm.websphere.rest.RenderUtils.renderError(500, MessageUtils.message('RM08101', [targetUser.username]) , throwable);
    	    }else{
                return com.ibm.websphere.rest.RenderUtils.renderError(500, MessageUtils.message('RM08101', [targetUser.username]) );
	        }
        }
        if(throwable){
            return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08007', throwable);
        }else{
            return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08007');
        }
        
    } else {
        // Audit that the password was successfully changed
        AuditDataHelper.audit(Action.UPDATE, "password" ,
		        MessageUtils.message('RM08071', [targetUser.username]));
        
        logger.INFO { MessageUtils.message('RM08071', [targetUser.username]) };
    }
    
    request.status = 204;
}
