/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.users.PermissionUtils;
import com.ibm.websphere.rest.RenderUtils;
import com.ibm.websphere.rest.RESTUtils;
import com.ibm.websphere.users.Groups;
import com.ibm.websphere.users.Users;

def onList()
{
    //if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
    if (!RESTUtils.authorized())
        return RenderUtils.renderError(403, 'RM08008');

    def roles;
    def userid = request.params.usersId[];
    def groupid = request.params.groupsId[];
    if (userid != null) {
        def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
        roles = PermissionUtils.getPermissionIds(id);
    } else if (groupid != null) {
        roles = Groups.getGroupRoles(groupid.toInteger());
    }

    request.view = RenderUtils.getRequestedView();
    request.json.output = roles;
    render();
}

def onPostMember()
{
    def session = user.session[];
    def role = request.params.rolesId[].toInteger();
    def resourceType = request.params.render.resourceType[];
    def userid = request.params.usersId[];
    def groupid = request.params.groupsId[];
    if (userid != null) {

        def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
        //if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN) || id == session.id)
        if (!RESTUtils.authorized() || id == session.id)
            return RenderUtils.renderError(403, 'RM08008');

        Users.addUserRole(id, role);
        invalidateSession(id);
    } else if (groupid != null) {
        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN))
            return RenderUtils.renderError(403, 'RM08008');

        // HACK: cannot add a role to 'Everyone' group
        if (groupid.toInteger() == 1) {
            return RenderUtils.renderError(400, 'RM08075');
        }
        Groups.addGroupRole(groupid.toInteger(), role);
        // invalidate all group members session except the current user session
        Groups.getGroupMembers(groupid.toInteger()).each { user ->
            if (user && user.id != session.id)
                invalidateSession(user.id)
        }
    }

    request.status = 201;
}

def onDelete()
{
    def session = user.session[];
    def role = request.params.rolesId[].toInteger();
    def userid = request.params.usersId[];
    def groupid = request.params.groupsId[];
    def success = true;
    if (userid != null) {
        def id = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();

        //if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN) || id == session.id || id == Permissions.SUPER_USER)
        if (!RESTUtils.authorized() || id == session.id || id == Permissions.SUPER_USER)
            return RenderUtils.renderError(403, 'RM08008');

        success = Users.deleteUserRole(id, role);
        invalidateSession(id);
    } else if (groupid != null) {
        if (!session.permissions.contains(Permissions.APPLIANCE_ADMIN))
            return RenderUtils.renderError(403, 'RM08008');

        // HACK: cannot remove a role from 'Everyone' group
        if (groupid.toInteger() == 1) {
            return RenderUtils.renderError(400, 'RM08076');
        }

        success = Groups.deleteGroupRole(groupid.toInteger(), role);
        // invalidate all group members session except the current user session
        Groups.getGroupMembers(groupid.toInteger()).each { user ->
            if (user && user.id != session.id)
                invalidateSession(user.id)
        }
    }

    request.status = success ? 204 : 404;
}
