/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.users;

import java.security.MessageDigest;

import com.ibm.websphere.rainmaker.core.SMTPSender;
import com.ibm.websphere.rainmaker.users.AuthenticatedUserCache;
import com.ibm.websphere.rainmaker.users.RainmakerUserService;
import zero.core.security.util.Base64;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.users.Users;

/**
 *
 * changePassword() returns:
 * 0 if the user's password was updated and email sent successfully
 * -1 if the user's password was changed but email was not sent
 * -2 if the user's password failed to change  
 *
 */
def changePassword(id, password)
{
    def salt = generateSalt();
    def hash = hashPassword(password, salt);
    def user = Users.getUser(id);
    if(id==1 && onAppliance()) {
    	// this is the super user that has an account on the appliance, 
    	// so need to change it there
    	if(!applianceChangePassword(user.username, password)) {
    		return -2;
    	}
    	if(config.security.collectiveSecretKey[]){
	    	def encryptedPwd = zero.core.security.encryption.SecretToken.encryptString(config.security.collectiveSecretKey[], password)
	    	zput(config.rainmaker.defaultPersistentZone[] + "/applianceUser",  encryptedPwd.toString()  )
	    	if (user.email) {
                if (_sendPasswordEmail(user, password)) {
                    return 0;
                } else {
                    return -1;
                }        
            }
    	}
    	return 0;
    }
    def success = updateResource('users', [id: id, salt: salt, passwordhash: hash]);
    
    if (success) {
        //
        // Remove from the authenticated user cache so that the user cannot login
        // with their old password - even though it would timeout relatively soon.
        //
        AuthenticatedUserCache.remove(user.username);
    
        if (user.email) {
           	if (_sendPasswordEmail(user, password)) {
           	    return 0;
           	} else {
           	    return -1;
           	}        
        }
    } else {
        return -2;
    }    
    return 0;
}

def _sendPasswordEmail(updatedUser, password)
{
    def lang = null;
    
    try
    {
        lang = user.lang[];
    }
    
    catch (Throwable t)
    {
        // only occurs if no HTTP request - this is a valid scenario
    }
    
    if (!lang)
        lang = MessageUtils.getLanguageFromLocale(Locale.getDefault());
    
    def success = false;
    try
    {
        success = SMTPSender.sendEmail(updatedUser.email, MessageUtils.message(lang, 'RM02001', []), """
${MessageUtils.message(lang, 'RM02007', [])}

${MessageUtils.message(lang, 'RM02003', [updatedUser.username])}
${MessageUtils.message(lang, 'RM02004', [password])}

${MessageUtils.message(lang, 'RM02006', [request.serverName[]])}
""");
    }
    
    catch (Throwable t)
    {
        logger.WARNING { MessageUtils.message(lang, 'RM02008', [updatedUser.email]) };
        return false;
    }
    
    return success;
}

/**
 *
 * generatePassword() returns a string of the given length, made up of random 
 * alpha-numeric and special characters.
 * 
 */
def generatePassword(length, withSpecialCharacters = true)
{
    def chars = "ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrtuvwxyz0123456789";
    def specialChars = "!@#%^*&-+=";
    
    if(withSpecialCharacters)
    {
    	chars = chars + specialChars;
    }	
    def numberOfChars = chars.length();
    
    def rand = new Random(System.currentTimeMillis());
    
    def password = [];
    
    for (i in 0..<length)
        password << chars[rand.nextInt() % numberOfChars];
    
    return password.join('');
}

/**
 * 
 * generateSalt() creates a pseudo-random value for use in password hashing. 
 * 
 */
def generateSalt()
{
	def salt = zero.core.security.encryption.SecretKeyGenerator.generateSecretKey();
    /*
    def obj = new Object();
    return obj.toString() + rand.nextInt();
     */
     return salt;
}

/**
 * 
 * hashPassword creates a SHA-1 digest of the string created by concatenating 
 * the given plain-text password and salt value. The resulting digest is 
 * re-encoded as a string before it is returned to the caller.
 * 
 */
def hashPassword(password, salt)
{
    def sha1 = MessageDigest.getInstance('SHA-1');
    def input = password + salt;
    def output = sha1.digest(input.getBytes("UTF-8"));
    return Base64.encode(output);
}

def login(user, password)
{        
    def userService = new RainmakerUserService();
    def results = userService.login(user, password);
    return !results.isEmpty();
}
