/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.datamgmt.widgets.QueryGrid2");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.form.FilteringSelect");
dojo.require("javascript.widgets.ModalDialog");
dojo.require("dijit.form.CheckBox");
dojo.require("javascript.widgets.MessageDialog");
dojo.require("javascript.widgets.XSFloatingPane");

dojo.requireLocalization("xsa.datamgmt.widgets", "xsa_messages");

dojo.declare("xsa.datamgmt.widgets.QueryGrid2", [dijit._Widget, dijit._Templated],{

	/* holds a reference to localized message bundle */
    xsa_messages : null,

	_maxResultSize: 500,
	_pageSize: 10,
	_currentPage: 0,
	_totalPages: 1,
	_sort: "none",
				
	templatePath: dojo.moduleUrl("xsa.datamgmt.widgets","templates/QueryGrid2Template.html"),
	widgetsInTemplate: true,

	_searchString: null,
	_to: null,
	_from: null,
	_dialog: null,
	_messageDialog: null,
	_error: null,
	
	_totalResults: null, //total number of keys that matched the query.  This may or may not be the same as the number of items in the table.
	
	_connections: [],
	
	_lastQuery: null,
	_queryId: null,
	standby:null,
	searchTableSummary:"",
	
	//The URLs can be overridden for testing
	gridURL: "/resources/griddata/0", 
	popFiltersURL: "/resources/griddata/0", 
	validateSearchURL: "/resources/griddata/0", 

	constructor: function(kwArgs) {
		if(!kwArgs) kwArgs = {};
		this.gridName = kwArgs.gridName;
		this.mapName = kwArgs.mapName;
		this._lastQuery = "";
		this._totalResults = null;
		
		if(kwArgs.gridURL) this.gridURL = kwArgs.gridURL;
		if(kwArgs.popFiltersURL) this.popFiltersURL = kwArgs.popFiltersURL;
		if(kwArgs.validateSearchURL) this.validateSearchURL = kwArgs.validateSearchURL;
		if(kwArgs.gridInUse) this.gridInUse = kwArgs.gridInUse;
		
		this.xsa_messages = dojo.i18n.getLocalization("xsa.datamgmt.widgets", "xsa_messages");
		this.xsa_mc_messages = dojo.i18n.getLocalization("xsa.messagecenter", "xsa_messages");
		
		this.searchLabel = this.xsa_messages.searchForKey;
		this.previous = this.xsa_messages.previousLabel;
		this.next = this.xsa_messages.nextLabel;
		this.searchTableMessage = this.xsa_messages.searchMessage;
		this.key = this.xsa_messages.key;
		this.value = this.xsa_messages.value;
		this.searchTableSummary = this.xsa_messages.searchTableSummary;
		this.pagination = this.xsa_messages.pagination;
	},
	
	postCreate: function() {	
		this.inherited(arguments);
		
		var populateFilters = function(result){ 
			if(!result.error) {
				this._from = 0;
				this._to = result.to.items[result.to.items.length-1].option;
			} else {
				this._errorMessage.innerHTML = result.error;
				dojo.style(this._errorMessage, "display", "");
				dojo.style(this._infoMessage, "display", "none");
				dojo.style(this.tableMessage, "display", "none");
				dojo.style(this.headerTable, "display", "none");
			}
		};
		                                                                
		var xhrArgs = {
			url: this.popFiltersURL,
			postData: dojo.toJson({action: "populatePartitionFilters", gridName:this.gridName, mapName:this.mapName}),
			sync: true,
			handleAs: "json",
			headers: {"Content-Type":"text/json"},
			load: dojo.hitch(this, populateFilters),
			error: dojo.hitch(this, function(error){
				this._errorMessage.innerHTML = this.xsa_messages.genericError;
				dojo.style(this._errorMessage, "display", "");
				dojo.style(this._infoMessage, "display", "none");
				dojo.style(this.tableMessage, "display", "none");
				dojo.style(this.headerTable, "display", "none");
			})
		};
		if(!this.gridInUse || this.mapName == "") {
			//No maps exist, so throw up a message
			this._infoMessage.innerHTML = this.xsa_messages.gridIsNotInUseInfo;
			dojo.style(this._infoMessage, "display", "");
			dojo.style(this._errorMessage, "display", "none");
			dojo.style(this.tableMessage, "display", "none");
			dojo.style(this.headerTable, "display", "none");
		} 
		if(this.mapName != "") {
			dojo.xhrPost(xhrArgs);
		}
		
		this.standby = new dojox.widget.Standby({target: this.tableData, duration:0});
		document.body.appendChild(this.standby.domNode);			              
    },
	
	resize: function(height, width){
		if(typeof height === "undefined") {
			height = this.lastHeight;
			width = this.lastWidth;
		} else {
			this.lastHeight = height;
			this.lastWidth = width;
		}
		
	},

	checkSearchEnter: function(e){
		if(e.keyCode === dojo.keys.ENTER) {
			this._searchString = e.target.value;
			this.runQuery(0);
			this._currentPage = 0;
		}
	},
		
	openRowDialog: function(rowId, e){
		dojo.stopEvent(e);
		
		//get the row information from the server
		var xhrArgs = {
				url: this.gridURL,
				postData: dojo.toJson({action: "getRow", rowId:rowId, query: this._lastQuery, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:this.mapName, queryId:this._queryId}),
				handleAs: "json",
				headers: {"Content-Type":"text/json"},
				load: dojo.hitch(this, function(response){
					if(response.error){
						dojo.style(this._errorMessage, "display", "");
						if(response.error=="query_lost"){
							this._errorMessage.innerHTML = this.xsa_messages.queryLostError;
						} else {
							this._errorMessage.innerHTML = this.xsa_messages.genericRetrieveError;
						}
					} else {
						var node = document.createElement('div');
							response.key = response.key.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
							response.value = response.value.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
							var displayValue = "<div style='padding:10px;'><span style='font-weight:bold'>"+this.xsa_messages.key+"</span><br><span style='word-wrap:break-word;'>"+response.key+"</span><br><br>";
							displayValue = displayValue + "<span style='font-weight:bold'>"+this.xsa_messages.value+"</span><br>";
							
							if(response.value.length > 22 && response.value.substring(0, 23) == "RestValue [contentType=") {
								var value = response.value.substring(response.value.indexOf("value=")+6, response.value.length-1);
								displayValue = displayValue + "<span style='word-wrap:break-word;'>"+value+"</span>";
							} else {
								displayValue = displayValue + "<span style='word-wrap:break-word;'>"+response.value+"</span>";	
							}
							
							getSessionAttributes = dojo.hitch(this, function(e){
								var xhrArgs = {
									url: this.gridURL,
									postData: dojo.toJson({action: "executeQuery", doNotSaveQuery:true, showFullKeyValue:true, sort:"key", query: response.key+".*", showValues:true, start:0, count:500, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:"objectgridSessionAttribute"}),
									handleAs: "json",
									headers: {"Content-Type":"text/json"},
									load: dojo.hitch(this, function(result){
										dojo.forEach(result.items, function(item){
											item.key = item.key.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
											attributeName = item.key.substring(item.key.indexOf(response.key)+response.key.length+1);
											var attributeTable = dojo.byId("sessionAttributeTable");
											var tr = dojo.create("tr",{});
											dojo.place(dojo.create("td",{innerHTML: attributeName, style:"padding:4px"}), tr, "first");
											dojo.place(dojo.create("td",{innerHTML: item.value, style:"padding:4px"}), tr, "last");
											dojo.place(tr, attributeTable, "last");
											dojo.style(attributeTable, "display", ""); 
										}, response);
										dojo.style(dojo.byId("showAttributesLink"), "display", "none");			
										document.body.style.cursor = "";
									}),
									error: function(error){
										document.body.style.cursor = "";
									}
								};
								document.body.style.cursor = "wait";
								dojo.xhrPost(xhrArgs);
							});
							
							if(this.mapName == "objectgridSessionMetadata"){
								displayValue = displayValue + "<br><br>";
								displayValue = displayValue + "<a href='javascript:getSessionAttributes();' id='showAttributesLink'>"+this.xsa_messages.showAttributes+"</a>";
								displayValue = displayValue + "<table class='cloudOE stable-striped clearfix' id='sessionAttributeTable' style='width:90%;display:none'>";
								displayValue = displayValue + "<thead><tr><th>"+this.xsa_messages.attribute+"</th>";
								displayValue = displayValue + "<th>"+this.xsa_messages.value+"</th></tr></thead>";
								displayValue = displayValue + "</table>";
								
							}
							
							displayValue = displayValue + "</div>";
							
							node.innerHTML = displayValue;
					
							dojo.place(node, this._resultsTable, "before");
							var tmp = new javascript.widgets.XSFloatingPane({
								title: "",
								dockable: false,
								closable: true,
								resizable: true
							},node);
							dojo.addClass(tmp.closeNode, "i-close-teal-bg-large");
							tmp.startup();
							dojo.attr(tmp.domNode, "aria-label", this.xsa_messages.fullKeyAndValueDialog);
							dojo.removeAttr(tmp.containerNode, "role");
							dojo.style(tmp.domNode, "padding", "0px");
							
							//determine the best size for the pane
							viewport = dojo.window.getBox();
							var paneWidth = 600;
							if(viewport.w < 600) paneWidth = viewport.w;
							pixPerChar = 7.5; //an estimate, it should get us close enough
							pixPerLine = 20; //an estimate, it should get us close enough
							
							//figure approx how many lines
							var linesPerKey = Math.ceil((response.key.length * pixPerChar)/paneWidth);
							var linesPerValue = Math.ceil((response.value.length * pixPerChar)/paneWidth);
							var totalLines = linesPerKey + linesPerValue + 5; //include headings and whitespace
							if(this.mapName == "objectgridSessionMetadata") totalLines = totalLines + 2;  // extra space for session attributes
							
							var paneHeight = totalLines*20;
							if(paneHeight > viewport.h) paneHeight = viewport.h;
							
							tmp.resize({ w:paneWidth, h:paneHeight });
							tmp.bringToTop();
							tmp.focusNode.focus();
					}
				}),
				error: function(error){
					dojo.style(this._errorMessage, "display", "");
					this._errorMessage.innerHTML = this.xsa_messages.genericRetrieveError;
				}
		};
		dojo.xhrPost(xhrArgs);
	},
	
	transformStringToRegEx: function(simpleQueryString) {
		if(simpleQueryString == ".*") return simpleQueryString;
		
		//First escape backslashes
		var patt = new RegExp("\\\\", "g");
		var regExString = simpleQueryString.replace(patt, "\\\\");
		
		var specialChartsPatt = new RegExp("[\\[\\]\\{\\}\\(\\)\\$\\.\\<\\>\\^\\-\\&\\+\\?\\,\\=]", "g");
		var regExString =regExString.replace(specialChartsPatt, "\\$&");

		var starsPatt = new RegExp("\\*", "g");
		var regExString = regExString.replace(starsPatt, ".*");
		
		if(regExString!="") regExString = ".*"+regExString+".*";
		console.log(regExString);
		return regExString;
	},
	
	runQuery: function(start){
		this.showStandby();
		
		this._lastQuery = this.transformStringToRegEx(this._searchString);
		var xhrArgs = {
				url: this.gridURL,
				postData: dojo.toJson({action: "executeQuery", sort:this._sort, query: this._lastQuery, showValues:true, start:start, count:this._pageSize, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:this.mapName, queryId:this._queryId}),
				handleAs: "json",
				headers: {"Content-Type":"text/json; charset=UTF-8"},
				sync: true,
				load: dojo.hitch(this, function(response){
					if(response.error){
						dojo.style(this._resultsTable, "display", "none");
						dojo.style(this._infoMessage, "display", "none");
						dojo.style(this._errorMessage, "display", "");
						dojo.style(this.tableMessage, "display", "none");
						if(response.error=="query_lost"){
							this._errorMessage.innerHTML = this.xsa_messages.queryLostError;			
						} else if(response.error=="invalidQuery") {
							this._errorMessage.innerHTML = this.xsa_messages.invalidQuery;
						} else {
							this._errorMessage.innerHTML = this.xsa_messages.genericQueryError+"&nbsp;"+this.xsa_messages.contactAdministrator;
						}
						this.standby.hide();
					} else {
						if(response.warning) { //This is rare and only happens if we get matches from one partition but fail to connect to one or more other partitions
							dojo.style(this._errorMessage, "display", "");
							this._errorMessage.innerHTML = response.warning;
						} else {
							dojo.style(this._errorMessage, "display", "none");
						}
						
						//destroy previous table
						dojo.query("tr[itemID]").forEach(dojo.destroy);
						
						if(response.queryId) {
							this._queryId = response.queryId;	
						} else {
							this._queryId = "";
						}
						
						if(response.items && response.items.length > 0) {
							dojo.style(this.tableMessage, "display", "none"); //Hide initial message telling the user to do a search
							dojo.style(this._resultsTable, "display", "");
							
							//build new rows
							dojo.forEach(response.items, function(item){
								var newRow = dojo.create("tr", {itemID:item.id}, this._resultsTableBody, "last");
								var key = item.key;
								if(key.length>50) key = key.substring(0,50)+"...";
							
								var keyTD = dojo.create("td", {headers:"keyHeader"}, newRow, "last");
								var link = dojo.create("a", {
									innerHTML: key,
								        href: "#",
									style: "color:#00aed1;font-weight:bold"
						                }, keyTD);
								this._connections.push(dojo.connect(link, "onclick", dojo.hitch(this, "openRowDialog", item.id)));
								item.value = item.value.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
								dojo.create("td", {innerHTML:item.value, headers:"valueHeader"}, newRow, "last");
							}, this);
					
							//show result footer
							var firstRowNumber = response.numRows > 0 ? this._pageSize*this._currentPage+1 : 0;
							var lastRowNumber = (this._pageSize*this._currentPage)+this._pageSize;
							lastRowNumber = (lastRowNumber > response.numRows) ? response.numRows : lastRowNumber;
						
							this._resultsTableIndexLabelDIV.innerHTML = firstRowNumber + "-" + lastRowNumber + " " + this.xsa_messages.of + " " + response.numRows;
							this._totalPages = Math.ceil(response.numRows/this._pageSize);
							
							//Add/remove previous, next, and divider
							dojo.style(this._pagingPrevious, "display", (this._currentPage>0) ? "" : "none");
							dojo.style(this._pagingNext, "display", (lastRowNumber < response.numRows) ? "" : "none");
							dojo.style(this._pagingDivider, "display", (this._currentPage>0 && lastRowNumber < response.numRows) ? "" : "none");
						} else {
							//No results found
							dojo.style(this.tableMessage, "display", "");
							dojo.style(this._resultsTable, "display", "none");
							this.tableMessage.innerHTML = this.xsa_messages.noResultsFound;	
						}
						
						if(response.totalMatches && response.totalMatches > response.numRows) {
							dojo.style(this._infoMessage, "display", "");
							this._infoMessageText.innerHTML = this.xsa_messages.maxResultLimitReachedWarningMsg2.replace("${_0_}",response.totalMatches).replace("${_1_}",this._maxResultSize);
						} else {
							dojo.style(this._infoMessage, "display", "none");
						}
						this.standby.hide();
					}
				}),
				error: function(error){
					dojo.style(this._resultsTable, "display", "none");
					dojo.style(this._errorMessage, "display", "");
					dojo.style(this._infoMessage, "display", "none");
					dojo.style(this.tableMessage, "display", "none");
					this._errorMessage.innerHTML = this.xsa_messages.genericQueryError+"&nbsp;"+this.xsa_messages.contactAdministrator;
					this.standby.hide();
				}
		};
		dojo.xhrPost(xhrArgs);
	},

	showStandby: function(){
		this.standby.show();
		dojo.style(this.standby._underlayNode, "opacity", 0.75)
		dojo.style(this.standby._imageNode, "opacity", 0.75)
	},
	                                                        	
	previousPage: function(){
		if(this._currentPage > 0) this._currentPage--;
		this.runQuery(this._currentPage*this._pageSize);
	},
	
	nextPage: function(){
		if(this._currentPage < this._totalPages) this._currentPage++;
		this.runQuery(this._currentPage*this._pageSize);
	},
	
	sortKey: function(){
		this.sortColumn("key");
		(this._sort=="key") ? 
			dojo.addClass(this.keyHeaderImg, "i-arrow-up-dark") :
			dojo.addClass(this.keyHeaderImg, "i-arrow-down-dark")
	},
	
	sortValue: function(){
		this.sortColumn("value");
		
		(this._sort=="value") ? 
			dojo.addClass(this.valueHeaderImg, "i-arrow-up-dark") :
			dojo.addClass(this.valueHeaderImg, "i-arrow-down-dark")
	},
	
	sortColumn: function(column){
		if(this._sort==column) { //sort descending
			this._sort = "-"+column;
		} else {
			this._sort = column;
		}
		//do sorting
		this.runQuery(0);
				
		//remove previous sort icons
		dojo.removeClass(this.valueHeaderImg, "i-arrow-up-dark");
		dojo.removeClass(this.valueHeaderImg, "i-arrow-down-dark");
		dojo.removeClass(this.keyHeaderImg, "i-arrow-up-dark");
		dojo.removeClass(this.keyHeaderImg, "i-arrow-down-dark");
	}, 

	clickSearch: function(e) {
		if(e.target.type!="text"){
			this._searchString = dojo.byId("searchInput").value;
			this.runQuery(0);
			this._currentPage = 0;
			this._queryTextBoxNode.focus();
		}
	},
	
	mouseEnter: function(){
		document.body.style.cursor = "pointer";	
	}, 
	
	mouseLeave: function(){
		document.body.style.cursor = "";
	},
	
	hideMessage: function(e){
		dojo.style(e.target.parentNode, "display", "none");
	}, 
		
	destroy: function(){
		//Cleanup connections
		dojo.forEach(this._connections, dojo.disconnect);
		if(this._selectAllFocusConnection) dojo.disconnect(this._selectAllFocusConnection);
		
		//Cleanup widgets
		if(this._queryTextBox) this._queryTextBox.destroyRecursive(true);
		if(this._searchButton) this._searchButton.destroyRecursive(true);
		if(this.standby) this.standby.destroyRecursive(true);
		
		delete this._queryTextBox;
		delete this._searchButton;
				
		this.inherited(arguments);
	}
	 	
});