/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.resizing.ResizingChart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("javascript.widgets.GenericDialog");
dojo.require("dojo.window");
dojo.require("dojox.timing");
dojo.require("dijit.Tooltip");

dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.resizing.ResizingChart", [dijit._Widget, dijit._Templated], {
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/resizing/templates/ResizingChartTemplate.html"),
	xsa_messages: null,
	
	gridName:null,
	theChart:null,
	_timer: null,
	
	chartType:"line",
	fill:false,
	url:null,
	aggregationScope:null,
	numberOfDatapoints:null,
	metrics:null,
	yAxisLabel:"",
	xAxisLabel:"",
	chartValueLabel:"",
	offset:null,
	overrideLabelKeys:null,
	locale: "en",
	
	constructor: function(args){
		this.gridName = args.gridName;
		this.chartArray = args.chartArray;
		this.aggregationScope = args.aggregationScope;
		this.metrics = args.metrics;

		if(args.yaxis) this.setYaxis(args.yaxis);
		if(args.chartType) this.chartType = args.chartType;
		if(args.fill) this.fill = args.fill;
		if(args.url) this.url = args.url;
		if(args.numberOfDatapoints) this.numberOfDatapoints = args.numberOfDatapoints;
		if(args.yAxisLabel) this.yAxisLabel = args.yAxisLabel;
		if(args.xAxisLabel) this.xAxisLabel = args.xAxisLabel;
		if(args.chartValueLabel) this.chartValueLabel = args.chartValueLabel;
		if(args.offset) this.offset = args.offset;
		if(args.overrideLabelKeys) this.overrideLabelKeys = args.overrideLabelKeys;
		
		this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
	},
	
	setYaxis: function(yaxis) {
		this.yaxis = yaxis;
		this.addTickFormatter();
		
	},
	
	addTickFormatter: function(){
		this.yaxis.tickFormatter = dojo.hitch( this, function (v, axis) {
			return dojo.number.format(v, { locale: this.locale});
		});
	},
	
	
	postCreate: function(){
		this.chart.id = this.id+"_chart";
		this.createTimer();
		this._timer.start();
		
		if(this.yAxisLabel) dojo.style(this.yAxisLabelDiv, "display", "");
		if(this.xAxisLabel) dojo.style(this.xAxisLabelDiv, "display", "");
		
		this.loadData(this.loadDataSuccess, null);
		
		if(this.chartType != "pie") {
			dojo.style(this.chartContainer, "paddingLeft", "25px");
			dojo.style(this.chartContainer, "paddingTop", "25px");
		}
		
		try{
			if(djConfig && djConfig.locale) this.locale = djConfig.locale;
		} catch (e) {
			console.log("Locale defaulting to english locale");
		}
	},
	
    createTimer: function() {
        this._timer = new dojox.timing.Timer(30000);
        this._timer.onTick = dojo.hitch(this, this._onTick);
    },
    
	_onTick: function(){
		this.loadData(this.loadDataUpdateSuccess, 1);

		if(this.theChart) {
			try {
				this.theChart.setData(this.chartArray);
				this.theChart.setupGrid();
				this.theChart.draw(); 
			} catch (e) {
				this._timer.stop();
			}
		} else {
			try {
				this.showChart();
			} catch (e) {
				this._timer.stop();
			}
		}
	},
    
	showErrorFunction:  function(message){
		
		//since chart isn't diplaying, hide the axis labels
		dojo.style(this.xAxisLabelDiv, "display", "none");
		dojo.style(this.yAxisLabelDiv, "display", "none");

		dojo.byId("errorMessageBanner").innerHTML = message;
		dojo.style(dojo.byId("errorMessageBanner"), "display", "");
	},
	
	resize: function(height, width){ 
		if(dojo.isIE && height.h) {
			width = height.w;
			height = height.h;
		}
		
		try {
			var yAxisLabelSize = dojo.marginBox(this.yAxisLabelDiv);
			var xAxisLabelSize = dojo.marginBox(this.xAxisLabelDiv);
			
			var height = height-xAxisLabelSize.w;
			var width =  width-yAxisLabelSize.h-15;
			
			dojo.style(this.chartContainer, "height", height+"px");
			dojo.style(this.chartContainer, "width", width+"px");
			
			if(this.yAxisLabel) {
				var chartPosition = dojo.coords(this.chart);
				
				if(dojo.isIE>9){
					dojo.style(this.yAxisLabelDiv, "left", chartPosition.x-(yAxisLabelSize.w/2)-10+"px");
				}

				if(dojo.isIE) { 
					dojo.style(this.yAxisLabelDiv, "top", chartPosition.y+chartPosition.h/2-(yAxisLabelSize.h/2)+"px");
					//Add width so long labels wrap
					if(chartPosition.h > 0  && yAxisLabelSize.h > chartPosition.h) {
						dojo.style(this.yAxisLabelDiv, "width", chartPosition.h+"px");
					} else {
						dojo.style(this.yAxisLabelDiv, "width", "");
					}
				} else {
					dojo.style(this.yAxisLabelDiv, "left", chartPosition.x-(yAxisLabelSize.w/2)-10+"px");
					dojo.style(this.yAxisLabelDiv, "top", chartPosition.y+chartPosition.h/2-(yAxisLabelSize.h/2)+"px");
					
					if(chartPosition.h > 0  && yAxisLabelSize.w > chartPosition.h) {
						dojo.style(this.yAxisLabelDiv, "width", chartPosition.h+"px");
					} else {
						dojo.style(this.yAxisLabelDiv, "width", "");
					}
				}
				
			}
			
			if(this.xAxisLabel) {
				var chartPosition = dojo.coords(this.chart);
				dojo.style(this.xAxisLabelDiv, "left", chartPosition.x+(chartPosition.w-40)/2+"px");
				dojo.style(this.xAxisLabelDiv, "top", chartPosition.y+chartPosition.h+5+"px");
			}
			if(!this.theChart) {
				try {
					this.showChart();
				} catch (e) {
					this.showErrorFunction(this.xsa_messages.ecaasStatsFailure);
					this._timer.stop();
				}
			}
				
			//Since the chart has absolute positioning we need to make the parent container stretch to the real
			//size of the chart, otherwise content around the chart will not render in the correct location.
			dojo.style(this.domNode.parentNode, "height", height+"px");
		} catch (e) {
			console.log(e);
		}
	},
	
	showChart: function(){
		if(this.chartArray && this.chartArray[0].data && this.chartArray[0].data[0]){ //Don't show the chart if we didnt get any data
		
			dojo.style(this.xAxisLabelDiv, "display", "");
			dojo.style(this.yAxisLabelDiv, "display", "");
			if(this.chartType=="line") {
				this.theChart = $.plot("#"+this.chart.id, this.chartArray, {
					xaxis: {
							mode: "time",
							tickLength: 0,
							timezone: "browser",
							minTickSize: [1, "minute"],
							tickFormatter: function (v, axis) {
                                return getTime(new Date(v), true);
                            }
						},
					yaxis: this.yaxis ? this.yaxis : 
						{
							tickLength: 0,
							tickFormatter: dojo.hitch( this, function (v, axis) {
							        return dojo.number.format(v, { locale: this.locale});
							})
						},
					series: {
						lines: {
							show: true,
							fill: this.fill
						},
						points: {
							show: true
						}
					},
					colors: ["#00849e"],
					grid: {
						hoverable: true,
						borderWidth: {top: 0, right: 0, bottom: 0, left: 0},
					}
				});

				var previousPoint = null;
				$("#"+this.chart.id).bind("plothover", dojo.hitch(this, function (event, pos, item) {
						if (item) {
							if (previousPoint != item.dataIndex) {

								previousPoint = item.dataIndex;

								$("#tooltip").remove();
								y = parseFloat(item.datapoint[1].toFixed(5));
						
								var newDate = new Date();
								newDate.setTime(item.datapoint[0]);
								var convertedTime = dojo.date.locale.format(newDate, {formatLength: "short", locale: this.locale});
								y = dojo.number.format(y, { locale: this.locale});
								
								this.showTooltip(item.pageX, item.pageY, y +"&nbsp;"+  this.chartValueLabel +"&nbsp;&nbsp;&nbsp;");
							}
						} else {
							if(this.dojoTooltip)this.dojoTooltip.destroyRecursive();
							$("#tooltip").remove();
							previousPoint = null;
						}
					
				}));
			} else if(this.chartType=="pie") {
					
				this.theChart = $.plot("#"+this.chart.id, this.chartArray, {
					series: {
						pie: {
							show: true,
							radius: 1,
							label: {
								show: true,
								radius: 1/2,
								formatter: dojo.hitch(this, this.labelFormatter, this.xsa_messages),
								background: {
									opacity: 0.5
								}
							}
						}
					},
					//colors: ["#195375", "#d4d4d4"],
					colors: ["#00849e", "#94a3ab"],
					legend: {
						show: true
					}
				});
			}
		} else { 
			//show error
			this.showErrorFunction(this.xsa_messages.ecaasStatsFailure);
		}
	},
	
	updatePieChartUnits: function(){
		
		if(this.chartArray[0] && this.chartArray[0].data && this.chartArray[0].data[0]){
		
		var value1 = this.chartArray[0].data[0][1];
		var value2 = this.chartArray[1].data[0][1];
			
		var value = (value1 > value2) ? value1 : value2;
		
		var offset = this.getUnitOffset(value);
		if(offset>0) {
			this.chartArray[0].data[0][1] = value1/(Math.pow(1024, offset));
			this.chartArray[1].data[0][1] = value2/(Math.pow(1024, offset));
		}
			
		this.chartArray[0].unitOffset = offset;
		
		var rawPercent1 = (value1/(value1+value2)) * 100;
		var rawPercent2 = (value2/(value1+value2)) * 100;
		var percent1 = rawPercent1.toFixed(0);
		var percent2 = rawPercent2.toFixed(0);
		
		if(percent1 + percent2 < 100) { 
			console.log("percent was not equal to 100, manually fixed it");
			(rawPercent1-percent1 > rawPercent2-percent2) ? percent1++ : percent2++; 
		}
		this.chartArray[0].percent = percent1;
		this.chartArray[1].percent = percent2;
		}
	},
	
	getUnitOffset: function(value){
		var offset = 0;
		for(offset = 0; value/1024 > 1; offset++) {
			value = value/1024;
		}
		return offset;                                                       
	},
	
	showTooltip: function(x, y, contents) { 
		if(this.dojoTooltip)this.dojoTooltip.destroyRecursive();
		
		$("<div id='tooltip'></div>").css({
				'white-space': "nowrap",
				position: "absolute",
				top: y,
				left: x-10
			}).appendTo("body");
		this.dojoTooltip = new dijit.Tooltip({
            				label: contents,
            				position: "above"
        			   });
		this.dojoTooltip.open(dojo.byId("tooltip")); 
	},
	
	labelFormatter: function(messages, label, series) {
		var value = series.data[0][1]==-1 ? 0 : series.data[0][1]; //Grid cap cannot be determined
		var offset = this.chartArray[0].unitOffset;	
		var units =  messages.bytes_abv;
		var percent = value == this.chartArray[0].data[0][1] ? this.chartArray[0].percent : this.chartArray[1].percent;
		var label = value == this.chartArray[0].data[0][1] ? this.chartArray[0].labelName : this.chartArray[1].labelName;
		
		if(value*10 >= 1 || value*100 >= 1) {
			value = parseFloat(value.toFixed(2));
		} else if(value>0){
			value = 0.01; // round it up to the hundreth.  
		} 
	
		units = this.getLabelUnitsFromOffset(offset, messages);	
		
		value = dojo.number.format(value, { locale: this.locale});
			
		return "<div class='pieChartLabel'>" + percent +"%&nbsp;("+value.toString() +"&nbsp;"+units+")<br>"+label+"&nbsp;</div>";
	},
	
	getLabelUnitsFromOffset: function(offset, messages){
		var units =  messages.bytes_abv;
		
				switch(offset) {
					case 0:
						units = messages.bytes_abv;
						break;
					case 1:
						units = messages.kbytes_abv;
						break;
					case 2:
						units = messages.mbytes_abv;
						break;
					case 3:
						units = messages.gbytes_abv;
						break;
					case 4:
						units = messages.tbytes_abv;
						break;
					case 5:
						units = messages.pbytes_abv;
						break;
					case 6:
						units = messages.ebytes_abv;
						break;
					case 7:
						units = messages.zbytes_abv;
						break;
				}
		return units;	
	},
	
	loadData: function(successFunction, numberOfDatapoints) {
		if (this.url == null) {
			dojo.publish(this.id+"_chartArrayUpdated", [{
			   chartArray:this.chartArray[0].data
			}]);
			return this.chartArray;
		}
		
		var deferred = dojo.xhrGet({
			url: this.url+"/"+this.aggregationScope,
			handleAs: "json",
			sync: true,
			preventCache: true,
			content: this.getRequestParameters(numberOfDatapoints),
			headers: { 'Accept': 'application/json' },
			load: dojo.hitch(this, successFunction),
			error: dojo.hitch(this, function(error) {
				
				if(numberOfDatapoints==null || numberOfDatapoints!=1) {// don't show error message on updates
					this.showErrorFunction(this.xsa_messages.ecaasStatsFailure);				
					
					dojo.publish("showUpdateMessage", [{
						message: this.xsa_messages.updateFailedMessage.replace("${0}", "<spam>"+getCurrentTime()+"</span>"),
						success: false
					}]);
				}
			})
		});
	},
	
	loadDataSuccess: function(response, ioArgs) {
		var tmpChartArray = dojo.fromJson(response);
		
		if(tmpChartArray[0] && tmpChartArray[0].data && tmpChartArray[0].data[0]) {
						
			this.chartArray = tmpChartArray;
			
			var index = 0;
			dojo.forEach(this.chartArray, function(point){
				if (this.overrideLabelKeys != null && this.overrideLabelKeys[index]) point.label = this.overrideLabelKeys[index];
				index++;
				if (point.label) {
					point.label = point.label.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
					point.label = eval("this.xsa_messages."+point.label); //translate labels
				}
			}, this);
			
			this.chartArray[0].labelName = this.chartArray[0].label.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
			this.chartArray[0].label = null;
			
			dojo.style(dojo.byId("errorMessageBanner"), "display", "none");
		
			if(this.chartType == "pie") {
				this.updatePieChartUnits();			
				this.chartArray[1].labelName = this.chartArray[1].label.split('<').join("&lt;");//encode brackets so it wont run xml, html, or javascript
				this.chartArray[1].label=null;
			} else {
				//remove the data points > 1 hour
				var dataArray = this.chartArray[0].data;
				if(dataArray.length!=0 &&  dataArray[dataArray.length-1][0] ) {
					var lastTimestamp = dataArray[dataArray.length-1][0];
					this.removeOldData(dataArray, lastTimestamp, this.numberOfDatapoints/2);
				}
			}
			
			if(this.metrics == "hitRate") {
				dojo.forEach(this.chartArray[0].data, function(dataPoint){dataPoint[1]=dataPoint[1]*100}, this);
			} else if(this.isMetricOffsetable(this.metrics) && this.offset && this.offset != "undefined") {
				if(this.offset > 0) {
					 dojo.forEach(this.chartArray[0].data, function(data){
				 		data[1] = data[1] / (Math.pow(1024, this.offset));
					 }, this);
				}
				this.chartValueLabel = this.getLabelUnitsFromOffset(this.offset, this.xsa_messages);
				this.yAxisLabelDiv.innerHTML = this.chartValueLabel;
			}
		
			dojo.publish(this.id+"_chartArrayUpdated", [{
			   chartArray:this.chartArray[0].data,
			   yAxisLabel: (this.isMetricOffsetable(this.metrics) && this.offset) ? this.getLabelUnitsFromOffset(this.offset, this.xsa_messages) : this.yAxisLabel
			}]);
			
		}
	},
	
	loadDataUpdateSuccess: function(response, ioArgs) {
		
		if (this.chartType == "pie")
			return this.loadDataSuccess(response, ioArgs);
		
		var responseArray = dojo.fromJson(response);

		if(responseArray[0] && responseArray[0].data[0]) {
			responseArray[0].label = null;
			
			dojo.style(dojo.byId("errorMessageBanner"), "display", "none");
			
			// Need to make sure we don't remove data point until the chart is ready to be pruned.
			var max = 120;
			if (this.numberOfDatapoints != null) {
				max = this.numberOfDatapoints;
			}
		
			for (var i=0; i<this.chartArray.length; i++) {
				var dataArray = this.chartArray[i].data;
				
				if(dataArray.length!=0 &&  dataArray[dataArray.length-1][0] != responseArray[i].data[0][0]) {
					//Remove data that is older than 1 hour
					var lastTimestamp = responseArray[i].data[0][0];
					this.removeOldData(dataArray, lastTimestamp, this.numberOfDatapoints/2);
				
					if(this.metrics == "hitRate") {
						responseArray[i].data[0][1] = responseArray[i].data[0][1]*100;
					} else if(this.isMetricOffsetable(this.metrics) && this.offset && this.offset != "undefined") {
						responseArray[i].data[0][1] = responseArray[i].data[0][1]/ (Math.pow(1024, this.offset));
					}

					// Need to check to make sure we aren't adding the same datapoint or an older datapoint.				
					if(responseArray[i].data[0][0] > dataArray[dataArray.length-1][0]){
						dataArray.push(responseArray[i].data[0]);
						this.chartArray[i].data = dataArray;
					}
					
					if (dataArray.length > max) {
						dataArray.shift();
					}
				}
			}
			
			if(this.isMetricOffsetable(this.metrics) && this.offset) {
				this.chartValueLabel = this.getLabelUnitsFromOffset(this.offset, this.xsa_messages);
			        this.yAxisLabelDiv.innerHTML = this.chartValueLabel;
			}
			
			dojo.publish(this.id+"_chartArrayUpdated", [{
		   		chartArray:this.chartArray[0].data,
				yAxisLabel: (this.isMetricOffsetable(this.metrics) && this.offset) ? this.getLabelUnitsFromOffset(this.offset, this.xsa_messages) : this.yAxisLabel
			}]);
		
			dojo.publish("showUpdateMessage", [{
				message: this.xsa_messages.updateMessage.replace("${0}", "<span>"+getCurrentTime()+"</span>"),
				success: true
			}]);
		} else {
			dojo.publish("showUpdateMessage", [{
				message: this.xsa_messages.updateFailedMessage.replace("${0}", "<span>"+getCurrentTime()+"</span>"),
				success: false
			}]);
		}
	},
	
	isMetricOffsetable: function(metricName) {
		return (metricName == "usedBytes" || metricName == "primaryUsedBytes");
	},
	
	removeOldData: function(array, lastTimestamp, minutes) {
		if(!minutes) minutes = 60;
		var numShifts = 0;
		dojo.forEach(array, function(data){
			if(data[0] < lastTimestamp-(60000*minutes)) {
				numShifts++;
			}
		}, this);
		for(var j=0; j<numShifts; j++) {
			array.shift();
		}
	},
	
	getRequestParameters: function(numberOfDatapoints) {
		var content;
		
		var localNumberOfDatapoints = numberOfDatapoints;
		if (localNumberOfDatapoints == null)
			localNumberOfDatapoints = this.numberOfDatapoints;

		if (localNumberOfDatapoints == null) {
			content = { 
				scope: this.aggregationScope,
				resourceName: this.gridName,
				metrics: this.metrics
			}
		} else {
			content = { 
				numberOfDatapoints: localNumberOfDatapoints,
				scope: this.aggregationScope,
				resourceName: this.gridName,
				metrics: this.metrics
			}
		}
		return content;
	}, 
	
	detroy: function() {
		if(this.theChart) this.theChart.shutdown();
		if(this.dojoTooltip)this.dojoTooltip.destroyRecursive();
		
		this.inherited(arguments);
	}
});
